import time

from netforce.model import Model, fields, get_model
from netforce.access import get_active_company

class LaborCostEntryLine(Model):
    _name="clinic.labor.cost.entry.line"
    _string="Labor Cost Entry Line"
    _multi_company=True
    
    def _get_all(self,ids,context={}):
        res={}
        for obj in self.browse(ids):
            staff=obj.staff_id
            cmax=staff.max_cycle or 0
            qty=obj.qty or 0
            over=min(0,cmax-qty)
            if not cmax: #XXX
                over=0
            res[obj.id]={
                'max_cycle': cmax,
                'over_cycle': over,
                'report_staff_id2': obj.report_staff_id.id, #XXX show PT
            }
        return res

    _fields={
        'entry_id': fields.Many2One("clinic.labor.cost.entry","Entry",required=True,on_delete="cascade"),
        'staff_id': fields.Many2One("clinic.staff","Staff",required=True),
        'date': fields.Date("Date"),
        'qty': fields.Integer("#Cycle"), #Cycle Item XXX
        'rate': fields.Float("Rate Avg."),
        'amount': fields.Float("Amount"),
        'company_id': fields.Many2One("company","Company"),
        'department_id': fields.Many2One("clinic.department","Department"),
        "type": fields.Selection([['staff','Staff'],["doctor","Doctor"],["nurse","Nurse"]],"Type"),
        "state": fields.Selection([["part_time","Part Time"],["full_time","Full Time"]],"Working Status",search=True),
        'report_staff_id': fields.Many2One("clinic.report.staff","#HDCase"),
        'max_cycle': fields.Integer("Max Cycle", function="_get_all",function_multi=True),
        'over_cycle': fields.Integer("Over Cycle", function="_get_all",function_multi=True),
        'report_staff_id2': fields.Many2One("clinic.report.staff","#PT",function="_get_all",function_multi=True),
        'categ_id': fields.Many2One("clinic.staff.categ",'Category'),
    }
    
    _order="type"

    _defaults={
        'date': lambda *a: time.strftime("%Y-%m-%d"),
        'company_id': lambda *a: get_active_company(),
    }

    def create(self,vals,**kw):
        staff=get_model('clinic.staff').browse(vals['staff_id'])
        vals['type']=staff.type
        new_id=super().create(vals,**kw)
        return new_id
    
    def delete(self,ids):
        for obj in self.browse(ids):
            report=obj.report_staff_id
            if report:
                report.delete()
        super().delete(ids)

LaborCostEntryLine.register()
