import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company


class ReportHDCaseDetail(Model):
    _name="clinic.report.hd.case.detail"
    _string="Report HDCase Detail"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'patient_type_id': fields.Many2One("clinic.patient.type","Payers"),
        'patient_id': fields.Many2One("clinic.patient","Patient"),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
        'department_id': fields.Many2One("clinic.department","Department"),
    }

    def _get_date_from(self,context={}):
        return time.strftime("%Y-%m-%d")

    def _get_date_to(self,context={}):
        return time.strftime("%Y-%m-%d")

    _defaults={
        'date': lambda *a: time.strftime("%Y-%m-%d"),
        'date_from': _get_date_from,
        'date_to': _get_date_to,
    }
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model('company').browse(company_id)
        date_from=time.strftime("%Y-%m-%d")
        date_to=time.strftime("%Y-%m-%d")
        ptype_id=None
        pid=None
        dpt_id=None
        bid=None
        if ids:
            obj=self.browse(ids)[0]
            date_from=obj.date_from
            date_to=obj.date_to
            pid=obj.patient_id.id
            ptype_id=obj.patient_type_id.id
            dpt_id=obj.department_id.id
            bid=obj.branch_id.id
        lines=[]
        dom=[]
        dom.append(['date','>=',date_from])
        dom.append(['date','<=',date_to])
        if pid:
            dom.append(['patient_id','=',pid])
        if ptype_id:
            dom.append(['patient_type_id','=',ptype_id])
        if dpt_id:
            dom.append(['department_id','=',dpt_id])
        if bid:
            dom.append(['branch_id','=',bid])

        draft=0
        wait_treat=0
        wait_pay=0
        in_progress=0
        finish_treat=0
        cancel=0
        paid=0
        no=1
        for hdcase in get_model("clinic.hd.case").search_browse(dom):
            state=hdcase.state
            if state=='in_progress':
                in_progress+=1
            elif state=='paid':
                paid+=1
            elif state=='cancelled':
                cancel+=1
            elif state=='waiting_treatment':
                wait_treat+=1
            elif state=='waiting_payment':
                wait_pay+=1
            elif state=='completed':
                finish_treat+=1
            else:
                draft+=1
                
            epo=[]
            fee_amt=0
            dlz_new='N'
            for dlz in hdcase.dialyzers:
                if dlz.use_time==1:
                    dlz_new='Y'
            for line in hdcase.lines:
                prod=line.product_id
                categ=line.product_categ_id
                if categ.code=='EPO':
                    epo.append(line.description or '')
                elif categ.code=='FEE':
                    fee_amt+=line.amount 
            if not epo:
                epo='-'
            else:
                epo=', '.join([e for e in epo])
            patient=hdcase.patient_id
            rmb_amt=hdcase.rmb_amount or 0
            more_amt=hdcase.due_amount or 0
            rmb=0
            if rmb_amt > 0:
                rmb=1
            lines.append({
                'no': no,
                'number': hdcase.number or '',
                'hd_case_id': hdcase.id or '',
                'time_start': hdcase.time_start,
                'epo': epo,
                'patient_name': patient.name,
                'patient_id': patient.id,
                'patient_number': patient.number or '',
                'patient_type': patient.type_id.name or '',
                'hct': hdcase.hct or 0,
                'rmb': rmb,
                'more_amt': more_amt,
                'fee_amt': fee_amt,
                'dlz_new': dlz_new,
            })
            no+=1
        
        sub_title=''
        if bid:
            sub_title+="%s "%(obj.branch_id.name or "")
        if dpt_id:
            sub_title+="%s "%(obj.department_id.name or "")
        data={
            'company_name': company.name or "",
            'parent_company_name': company.parent_id.name or "",
            'lines': lines,
            'date_from': date_from,
            'date_to': date_to,
            'sub_title': sub_title,
            'draft': draft,
            'wait_treat': wait_treat,
            'wait_pay': wait_pay,
            'in_progress': in_progress,
            'cancel': cancel,
            'paid': paid,
            'finish_treat': finish_treat,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_date_from(self,context={}):
        data=context['data']
        data['date_to']=data['date_from']
        return data

ReportHDCaseDetail.register()
